/*
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * Copyright (C) 2007 Dr. Ruediger Berlich - ruediger@berlich.de
 */

// Compile with "g++ -o bind2 bind2.cpp"
// Tested on OpenSUSE 10.2, g++ 4.12, boost 1.33.1

#include <boost/shared_ptr.hpp>
#include <boost/bind.hpp>

#include <functional>
#include <iostream>
#include <vector>


using namespace std;
using namespace boost;

/******************************************************************************/

class payload{
public:
    payload(const string& id){
	id_ = id;
    }

    ~payload(){
	cout << "Destructor of object" << id_ << endl;
    }

    void printObjectId(void){
	cout << "Called for object" << id_ << endl;
    }

    string getObjectId(void){
	return id_;
    }
    
private:
    string id_;
};

/******************************************************************************/

shared_ptr<payload> payloadFactory(const string& id){
    payload *retVal = new payload(id);
    shared_ptr<payload> retValPtr(retVal);
    return retValPtr;
}

/******************************************************************************/

int main(int argc, char **argv){
    vector<shared_ptr<payload> > payloadContainer;

    // payloadFactory erzeugt einen shared_ptr<payload>
    payloadContainer.push_back(payloadFactory("A"));
    payloadContainer.push_back(payloadFactory("B"));
    payloadContainer.push_back(payloadFactory("C"));

    sort(payloadContainer.begin(),
         payloadContainer.end(),
         bind(
            std::greater<string>(),
            bind(&payload::getObjectId, _1),
            bind(&payload::getObjectId, _2)
         )
    );

    for_each(payloadContainer.begin(), 
             payloadContainer.end(), 
             bind(&payload::printObjectId,_1));
}
 
/******************************************************************************/

