/*
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * Copyright (C) 2007 Dr. Ruediger Berlich - ruediger@berlich.de
 */

// Compile with "g++ -o operators1 operators1.cpp"
// Tested on OpenSUSE 10.2, g++ 4.12, boost 1.33.1

#include <boost/operators.hpp>

#include <iostream>

using namespace std;
using namespace boost;

/******************************************************************************/

class myInt
    :boost::operators<myInt>
{
public:
    myInt(void){
	myVal_ = 0;
    }

    myInt(int myVal){
	myVal_ = myVal;
    }

    myInt(const myInt& cp){
	myVal_ = cp.myVal_;
    }

    int value(void) const{
	return myVal_;
    }

    /********** Operators **********/

    myInt& operator=(const myInt& cp){
	myVal_ = cp.myVal_;
	return *this;
    }
    
    bool operator<(const myInt& x) const{
	return myVal_ < x.myVal_;
    }

    bool operator==(const myInt& x) const{
	return myVal_ == x.myVal_;
    }

    myInt& operator+=(const myInt& x){
	myVal_ += x.myVal_;
	return *this;
    }

    myInt& operator-=(const myInt& x){	
	myVal_ -= x.myVal_;
	return *this;	
    }

    myInt& operator*=(const myInt& x){
	myVal_ += x.myVal_;
	return *this;
    }

    myInt& operator/=(const myInt& x){
	if(x.myVal_) myVal_ /= x.myVal_;
	return *this;
    }

    myInt& operator%=(const myInt& x){
	myVal_%=x.myVal_;
	return *this;
    }

    myInt& operator|=(const myInt& x){
	myVal_|=x.myVal_;
	return *this;	
    }

    myInt& operator&=(const myInt& x){
	myVal_&=x.myVal_;
	return *this;		
    }

    myInt& operator^=(const myInt& x){
	myVal_^=x.myVal_;
	return *this;	
    }

    myInt& operator++(){
	myVal_ += 1;
	return *this;
    }

    myInt& operator--(){
	myVal_ -= 1;
	return *this;
    }

private:
    int myVal_;
};

/******************************************************************************/

int main(int argc, char** argv){
    myInt a(1), b(2), c=a+b;

    cout << "a = " << a.value() << endl
	 << "b = " << b.value() << endl
	 << "c = " << c.value() << endl;

    if(a<=b) cout << "a is smaller or equal b" << endl;
    if(a!=b) cout << "a is not equal to b" << endl;
    if(c>b)  cout << "c is larger than b" << endl;
}

/******************************************************************************/

